# -*- coding: utf-8 -*-

#changing the diefault directory
import os
os.chdir("... set your directory ...")

#data importation
import pandas
image_all = pandas.read_table("image.txt",sep="\t",header=0,decimal=".")

#shape of the table (data frame)
print(image_all.shape)

#name of variables
print(image_all.columns)

#type of the variables
print(image_all.dtypes)

#print the 5 first rows
print(image_all.head())

#5 first rows - the first and last columns
print(image_all.iloc[:5,[0,20]])

#partitioning into training and test samples

#training set - select the corresponding rows using "sample" column
image_train = image_all[image_all["sample"]=="train"]
#remove the "sample" column
image_train = image_train.iloc[:,0:20]
#checking
print(image_train.shape)
print(image_train.head())
#transformation into a NumPy matrix
d_train = image_train.as_matrix()
y_app = d_train[:,0]
X_app = d_train[:,1:20]

#test sample
image_test = (image_all[image_all["sample"]=="test"]).iloc[:,0:20]
print(image_test.shape)
y_test = image_test.as_matrix()[:,0]
X_test = image_test.as_matrix()[:,1:20]

#module for the evaluation of classifiers
from sklearn import metrics

#function for the calculation of the error rate
#modele = classifier object
def error_rate(modele,y_test,X_test):
    #prediction
    y_pred = modele.predict(X_test)
    #error rate
    err = 1.0 - metrics.accuracy_score(y_test,y_pred)
    #return
    return err
#end fonction
    
#classification tree class
from sklearn.tree import DecisionTreeClassifier

#instantiation
dtree = DecisionTreeClassifier()

#settings of the tree
print(dtree)

#learning phase
dtree.fit(X_app,y_app)

#***display the decision tree

#generate the output in the ".dot" format
from sklearn import tree
tree.export_graphviz(dtree,out_file="tree.dot",feature_names=image_train.columns[1:20],class_names=image_train.columns[0])

#importance of variables - 0 when the variable is not included into the tree
imp = {"VarName":image_train.columns[1:],"Importance":dtree.feature_importances_}
print(pandas.DataFrame(imp))

#error rate
print(error_rate(dtree,y_test,X_test))

#bagging class
from sklearn.ensemble import BaggingClassifier

#instantiation
baggingTree = BaggingClassifier(DecisionTreeClassifier(),n_estimators=20)
print(baggingTree)

#fit the model on the learning set
baggingTree.fit(X_app,y_app)

#evaluation
print(error_rate(baggingTree,y_test,X_test))

#experiment - tree number

#function - training / test for a given m - bagging approach
def train_test_bagging(m,X_app,y_app,X_test,y_test):
    bag = BaggingClassifier(DecisionTreeClassifier(),n_estimators=m)
    bag.fit(X_app,y_app)
    return error_rate(bag,y_test,X_test)
#end train-test

#values of m to evaluate
m_a_tester = [1,5,10,20,50,100,200]

#matrix of results
import numpy
result = numpy.zeros(shape=(1,7))

#repeat 20 times the experiment for each m
for expe in range(20):
    #evaluate each value of m
    res = [train_test_bagging(m,X_app,y_app,X_test,y_test) for m in m_a_tester]
    res = numpy.asarray(res).reshape(1,7)
    #add a new row into the matrix
    result = numpy.append(result,res,axis=0)
#

#remove the first row
result = numpy.delete(result,0,axis=0)

#calculate the mean
mresult = numpy.mean(result,axis=0)
print(mresult)

#chart
import matplotlib.pyplot as plt

#label of the axes
plt.xlabel("m")
plt.ylabel("Err. Rate")

plt.plot(m_a_tester,mresult,linewidth=2)

#grid search tool
#using cross-validation
from sklearn.grid_search import GridSearchCV

# the parameters to make vary
# the name of the parameter must be explicit
# we enumerate the values to try
parametres = [{"n_estimators":[1,5,10,20,50,100,200]}]

#instantiate the classifier
bag = BaggingClassifier(DecisionTreeClassifier())

#instantiation of the gris search tool
#the metric used is the accuracy rate (error rate = 1 - accuracy rate)
grid_bag = GridSearchCV(estimator=bag,param_grid=parametres,scoring="accuracy")

#launching the exploration
grille_bag = grid_bag.fit(X_app,y_app)

#print the results
print(grille_bag.grid_scores_)

#best score
print(grille_bag.best_score_)

#parameter for the best score
print(grille_bag.best_params_)

#evaluation of the best classifier on the test set
print(error_rate(grille_bag,y_test,X_test))

#linear discriminant analysis class
from sklearn.discriminant_analysis import LinearDiscriminantAnalysis

#instantiation - bagging of linear discriminant analysis
bag_lda = BaggingClassifier(LinearDiscriminantAnalysis(),n_estimators=20)
print(bag_lda)

#learning phase&
bag_lda.fit(X_app,y_app)

#evaluation
print(error_rate(bag_lda,y_test,X_test))

#linear discriminant analysis
lda = LinearDiscriminantAnalysis()
lda.fit(X_app,y_app)
print(error_rate(lda,y_test,X_test))


#RandomForest class
from sklearn.ensemble import RandomForestClassifier

#instantiation
rf = RandomForestClassifier(n_estimators=20)

#learning phase
rf.fit(X_app,y_app)

#test error rate
print(error_rate(rf,y_test,X_test))

#importance of variables
print(rf.feature_importances_)

#with the name of the variables
imp = {"VarName":image_train.columns[1:],"Importance":rf.feature_importances_}
print(pandas.DataFrame(imp))

#search the "optimal" number of trees
from sklearn.grid_search import GridSearchCV

#tree number to test
parametres = [{"n_estimators":[1,5,10,20,50,100,200]}]

#instantiation
grid_rf = GridSearchCV(estimator=rf,param_grid=parametres,scoring="accuracy")

#lauching the exploration
grille_rf = grid_rf.fit(X_app,y_app)

#print the results
print(grille_rf.grid_scores_)

#best score
print(grille_rf.best_score_)

#best setting
print(grille_rf.best_params_)

#evaluation on the test set
print(error_rate(grille_rf,y_test,X_test))

#Adaboost
from sklearn.ensemble import AdaBoostClassifier

#instantiation
ab = AdaBoostClassifier(algorithm="SAMME",n_estimators=20, base_estimator = DecisionTreeClassifier())
print(ab)

#fitting on the training set
ab.fit(X_app,y_app)

#evaluation
print(error_rate(ab,y_test,X_test))
