#clear the memory
rm(list=ls())

#data importation
setwd(" ... the directory of your dataset ...")
image_all <- read.table("image.txt",sep="\t",dec=".",header=TRUE)
print(summary(image_all))

#parititoning into training and test samples
#using the 'sample' column
image_train <- image_all[image_all$sample=="train",1:20]
image_test <- image_all[image_all$sample=="test",1:20]

print(summary(image_train$REGION_TYPE))
print(summary(image_test$REGION_TYPE))

#function for performance evaluation
#error rate = 1 - accuracy rate
error_rate <- function(yobs,ypred){
  #confusion matrix
  mc <- table(yobs,ypred)
  #error rate
  err <- 1.0 - sum(diag(mc))/sum(mc)
  return(err)
}

#classification tree
library(rpart)
arbre_1 <- rpart(REGION_TYPE ~ ., data = image_train)
print(arbre_1)

#prediction on the test set
pred_1 <- predict(arbre_1,newdata=image_test,type="class")

#test error rate
print(error_rate(image_test$REGION_TYPE,pred_1))

#decision stump
param_stump = rpart.control(cp=0,maxdepth=1,minsplit=2,minbucket=1)
arbre_2 <- rpart(REGION_TYPE ~ ., data = image_train,control=param_stump)
print(arbre_2)

#prediction and error rate
pred_2 <- predict(arbre_2,newdata=image_test,type="class")
print(error_rate(image_test$REGION_TYPE,pred_2))

#deep tree
param_deep = rpart.control(cp=0,maxdepth=30,minsplit=2,minbucket=1)
arbre_3 <- rpart(REGION_TYPE ~ ., data = image_train,control=param_deep)
print(arbre_3)

#prediction and error rate
pred_3 <- predict(arbre_3,newdata=image_test,type="class")
print(error_rate(image_test$REGION_TYPE,pred_3))

#library adabag
library(adabag)

#bagging
bag_1 <- bagging(REGION_TYPE ~ ., data = image_train,mfinal=20)

#prediction
predbag_1 <- predict(bag_1,newdata = image_test)

#error rate
print(error_rate(image_test$REGION_TYPE,predbag_1$class))

#accessing to the first tree
print(bag_1$trees[[1]])

#variable importance
print(sort(bag_1$importance,decreasing=TRUE))

#graphical representation
importanceplot(bag_1,cex.names=0.5,horiz=TRUE)

#bagging with only one tree
param = rpart.control(maxsurrogate=20)
bag_seul <- bagging(REGION_TYPE ~ ., data = image_train,mfinal=1)

#the tree
print(bag_seul$trees[[1]])

#variable importance
print(bag_seul$importance)

#bagging of decision stump
bag_stump <- bagging(REGION_TYPE ~ ., data = image_train,mfinal=20, control=param_stump)

#prediction
predbag_stump <- predict(bag_stump,newdata = image_test)

#error rate
print(error_rate(image_test$REGION_TYPE,predbag_stump$class))

#bagging deep tree
bag_deep <- bagging(REGION_TYPE ~ ., data = image_train,mfinal=20, control=param_deep)

#prediction
predbag_deep <- predict(bag_deep,newdata = image_test)

#error rate
print(error_rate(image_test$REGION_TYPE,predbag_deep$class))

#number of trees to try
m_a_tester <- c(1,5,10,20,50,100,200)

#function for training and testing bagging
#m is the number of trees
train_test_bag <- function(m){
  bag <- bagging(REGION_TYPE ~ .,data=image_train,mfinal=m,control=param_deep)
  predbag <- predict(bag,newdata = image_test)
  return(error_rate(image_test$REGION_TYPE,predbag$class))
}

#evaluate 20 times for each value of m
result <- replicate(20,sapply(m_a_tester,train_test_bag))

#chart
plot(m_a_tester,apply(result,1,mean),xlab="m",ylab="Err. rate",type="b")

#random forest
library(randomForest)
rf_1 <- randomForest(REGION_TYPE ~ ., data = image_train, ntree = 20)

#prediction
predrf_1 <- predict(rf_1,newdata=image_test,type="class")

#error rate
print(error_rate(image_test$REGION_TYPE,predrf_1))

#out-of-bag confusion matrix
print(rf_1$confusion)

#out-of-bag error rate estimate
print(1-sum(diag(rf_1$confusion))/sum(rf_1$confusion))

#appearances of variables
print(data.frame(cbind(colnames(image_train)[2:20],varUsed(rf_1))))

#importance of variables
varImpPlot(rf_1)

#accessing to the first tree
print(getTree(rf_1,1))

#number of trees to try
m_a_tester <- c(1,5,10,20,50,100,200)

#function for training and testing random forest
#m is the number of trees
train_test_rf <- function(m){
  rf <- randomForest(REGION_TYPE ~ .,data=image_train,ntree=m)
  predrf <- predict(rf,newdata = image_test)
  return(error_rate(image_test$REGION_TYPE,predrf))
}

#evaluate 20 times for each value of m
result <- replicate(20,sapply(m_a_tester,train_test_rf))

#chart
plot(m_a_tester,apply(result,1,mean),xlab="m",ylab="Err. rate",type="b")

#library adabag
library(adabag)

#boosting
bo_1 <- boosting(REGION_TYPE ~ ., data = image_train,mfinal = 20, boos= FALSE)

#prediction
predbo_1 <- predict(bo_1,newdata = image_test)

#error rate
print(error_rate(image_test$REGION_TYPE,predbo_1$class))

#boosting of decision stump
bo_stump <- boosting(REGION_TYPE ~ ., data = image_train,mfinal=20, boos=FALSE, coeflearn= 'Zhu', control=param_stump)

#prediction
predbo_stump <- predict(bo_stump,newdata = image_test)

#error rate
print(error_rate(image_test$REGION_TYPE,predbo_stump$class))

#boosting deep tree
bo_deep <- boosting(REGION_TYPE ~ ., data = image_train,mfinal=20, boos=FALSE, coeflearn= 'Zhu', control=param_deep)

#prediction
predbo_deep <- predict(bo_deep,newdata = image_test)

#error rate
print(error_rate(image_test$REGION_TYPE,predbo_deep$class))

#number of trees to try
m_a_tester <- c(1,5,10,20,50,100,200)

#function for training and testing random forest
#m is the number of trees
train_test_boosting <- function(m){
  bo <- boosting(REGION_TYPE ~ .,data=image_train,mfinal=m,coeflearn='Zhu')
  predbo <- predict(bo,newdata = image_test)
  return(error_rate(image_test$REGION_TYPE,predbo$class))
}

#evaluate 20 times for each value of m
result <- replicate(20,sapply(m_a_tester,train_test_boosting))

#chart
plot(m_a_tester,apply(result,1,mean),xlab="m",ylab="Err. rate",type="b")
